using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace ConvexHulls
{
    class AklToussaintHeuristic
    {
        public static Point[] PrunePoints(Point[] points)
        {
            // search for min and max x and y
            Point minx = points[0];
            Point maxx = points[0];
            Point miny = points[0];
            Point maxy = points[0];
            foreach (Point p in points)
            {
                if (p.X < minx.X)
                    minx = p;
                else if (p.X > maxx.X)
                    maxx = p;
                if (p.Y < miny.Y)
                    miny = p;
                else if (p.Y > maxy.Y)
                    maxy = p;
            }

            // prune points inside this quadrilateral
            bool[] pruned = new bool[points.Length];
            int count = 0;
            for(int i=0; i<points.Length; i++)
            {
                Point p = points[i];
                if (isInside(p, minx, maxx, miny, maxy))
                    pruned[i] = true;
                else
                    count++;
            }

            // calculate result array
            Point[] result = new Point[count];
            for (int i = 0, j = 0; i < points.Length; i++)
            {
                if (pruned[i] == false)
                    result[j++] = points[i];
            }

            return result;
        }

        public static List<Point> PrunePoints(List<Point> points)
        {
            // search for min and max x and y
            Point minx = points[0];
            Point maxx = points[0];
            Point miny = points[0];
            Point maxy = points[0];
            foreach(Point p in points)
            {
                if (p.X < minx.X)
                    minx = p;
                else if (p.X > maxx.X)
                    maxx = p;
                if (p.Y < miny.Y)
                    miny = p;
                else if (p.Y > maxy.Y)
                    maxy = p;
            }

            // prune points inside this quadrilateral
            List<Point> plist = new List<Point>();
            plist.Add(minx);
            if (maxx != minx)
                plist.Add(maxx);
            if (miny != minx && miny != maxx)
                plist.Add(miny);
            if (maxy != minx && maxy != maxx && maxy != miny)
                plist.Add(maxy);
            foreach (Point p in points)
            {
                if (!isInside(p, minx, maxx, miny, maxy))
                    plist.Add(p);
            }

            return plist;
        }

        private static bool isInside(Point p, Point minx, Point maxx, Point miny, Point maxy)
        {
            return (isPRight(p, minx, miny) && isPRight(p, miny, maxx)
                 && isPRight(p, maxx, maxy) && isPRight(p, maxy, minx));
        }

        /// <summary>
        /// Returns the distance of a point p to a vector (from,to).
        /// </summary>
        private static int getDist(Point p, Point vfrom, Point vto)
        {
            return ((vto.X - vfrom.X) * (p.Y - vfrom.Y) - (p.X - vfrom.X) * (vto.Y - vfrom.Y));
        }

        /// <summary>
        /// Returns true, if the point p is on the right hand side of the vector vfrom->vto.
        /// </summary>
        private static bool isPRight(Point p, Point vfrom, Point vto)
        {
            return (getDist(p, vfrom, vto) > 0);
        }
    }
}
