using System;
using System.Collections.Generic;
using System.Text;
using System.Drawing;

namespace Labyrinth
{
    class LabyrinthVisualisation
    {
        private Image image;
        /// <summary>
        /// The Labyrinth will be drawn on this image
        /// </summary>
        public Image Image
        {
            get { return image; }
        }

        private Labyrinth labyrinth;

        public Labyrinth Labyrinth
        {
            get { return labyrinth; }
            set { labyrinth = value; Draw();  }
        }

        private int roomSize = 10;
        /// <summary>
        /// Width and height of one room in the labyrinth in pixels
        /// </summary>
        public int RoomSize
        {
            get { return roomSize; }
        }	

        /// <summary>
        /// Draws the labyrinth on the image
        /// </summary>
        /// <param name="image">The image to draw on</param>
        /// <param name="labyrinth">The labyrinth data</param>
        public LabyrinthVisualisation(Image image, Labyrinth labyrinth)
        {
            this.image = image;
            this.Labyrinth = labyrinth;
        }

        /// <summary>
        /// Draws the labyrinth on the image
        /// </summary>
        /// <param name="image">The image to draw on</param>
        /// <param name="labyrinth">The labyrinth data</param>
        /// <param name="roomSize">The pixel width and height one room will have on the image</param>
        public LabyrinthVisualisation(Image image, Labyrinth labyrinth, int roomSize)
        {
            this.image = image;
            this.roomSize = roomSize;
            this.Labyrinth = labyrinth;
        }

        /// <summary>
        /// Redraws the labyrinth
        /// </summary>
        public void Draw()
        {
            Graphics g = Graphics.FromImage(image);
            g.FillRectangle(Brushes.White, g.ClipBounds);
            for (int i = 0; i < labyrinth.Rooms.GetLength(0); i++)
            {
                for (int j = 0; j < labyrinth.Rooms.GetLength(1); j++)
                {
                    Rectangle r = new Rectangle(i * roomSize, j * roomSize, roomSize, roomSize);
                    g.FillRectangle(labyrinth.Rooms[i, j].GetBrush(), r);
                    if (labyrinth.Rooms[i, j].North) g.DrawLine(Pens.Black, new Point(r.Left, r.Top), new Point(r.Right-1, r.Top));
                    if (labyrinth.Rooms[i, j].East) g.DrawLine(Pens.Black, new Point(r.Right-1, r.Top), new Point(r.Right-1, r.Bottom-1));
                    if (labyrinth.Rooms[i, j].South) g.DrawLine(Pens.Black, new Point(r.Left, r.Bottom-1), new Point(r.Right-1, r.Bottom-1));
                    if (labyrinth.Rooms[i, j].West) g.DrawLine(Pens.Black, new Point(r.Left, r.Top), new Point(r.Left, r.Bottom-1));
                }
            }
        }
    }
}
