using System;
using System.Collections.Generic;
using System.Text;
using System.IO;

namespace Labyrinth
{
    class StatisticExperiments
    {
        // general central parameters
        int runs = 50;       // number of runs per experiment
        int labruns = 10;    // runs over one labyrinth

        // central parameters used in method expVarSize()
        byte[] loosenesses = new byte[] { 0, 5, 30 };  // loosenesses for creating the different labyrinth types
        int maxSize = 2000;  // maximum size of a labyrinth
        int sdiff = 100;     // difference between two labyrinth sizes in the calculation

        // central parameters used in method expVarLooseness()
        byte maxLoose = 100; // maximum looseness
        byte ldiff = 5;      // difference between two looseness values in the measurement
        int lsize = 1000;    // labyrinth size to use

        // timer for time measure
        HiResTimer timer = new HiResTimer();


        public StatisticExperiments() { }

        /// <summary>
        /// Runs the defined experimants.
        /// </summary>
        public void Run()
        {
            System.Console.WriteLine("Starte Auswertung...");
            
            //expVarSize();
            expVarLooseness();

            System.Console.WriteLine("Auswertung beendet...");
        }

        /// <summary>
        /// Experiment for the path finding algorithms on variable labyrinth sizes and for the defined loosenesses.
        /// </summary>
        private void expVarSize()
        {
            System.Console.WriteLine("\n ===>  Starte Auswertung expVarSize()  <===");

            // measurement of time and marked rooms per labyrinth size for some defined loosenesses
            double[, ,] times = new double[loosenesses.Length, maxSize / sdiff, 2];
            double[, ,] markCounts = new double[loosenesses.Length, maxSize / sdiff, 2];
            long startTime, endTime;
            // for all defined loosenesses
            for (int l = 0; l < loosenesses.Length; l++)
            {
                System.Console.WriteLine("\n=> Lockerheit = " + loosenesses[l]);
                // for every labyrinth size
                for (int s = sdiff, i = 0; s <= maxSize; s += sdiff, i++)
                {
                    System.Console.WriteLine("-> Labyrinth Gre: " + s);
                    // go through all runs
                    for (int run = 1; run <= runs; run++)
                    {
                        System.Console.Write(run + ",");
                        // create labyrinth
                        Labyrinth lab = new Labyrinth(s, loosenesses[l]);
                        LabPathFinder finder = new LabPathFinder(lab);

                        // make some DFS runs over this labyrinth
                        for (int j = 0; j < labruns; j++)
                        {
                            lab.UnmarkRooms();
                            startTime = timer.Value;
                            finder.FindWayDFSIt();
                            endTime = timer.Value;
                            times[l, i, 0] += (endTime - startTime) / (double)timer.Frequency;
                            markCounts[l, i, 0] += lab.CountMarkedRooms();
                        }

                        // make some TurnCounter runs over this labyrinth
                        for (int j = 0; j < labruns; j++)
                        {
                            lab.UnmarkRooms();
                            startTime = timer.Value;
                            finder.FindWayTurnCounter();
                            endTime = timer.Value;
                            times[l, i, 1] += (endTime - startTime) / (double)timer.Frequency;
                            markCounts[l, i, 1] += lab.CountMarkedRooms();
                        }
                    }

                    // calculate mean values
                    times[l, i, 0] /= (double)(runs * labruns);
                    times[l, i, 1] /= (double)(runs * labruns);
                    markCounts[l, i, 0] /= (double)(runs * labruns);
                    markCounts[l, i, 1] /= (double)(runs * labruns);

                    System.Console.WriteLine();
                    System.Console.WriteLine("DFS:       " + times[l, i, 0].ToString().Replace(',', '.') + "s, " + markCounts[l, i, 0].ToString().Replace(',', '.') + " Markierungen");
                    System.Console.WriteLine("TurnCount: " + times[l, i, 1].ToString().Replace(',', '.') + "s, " + markCounts[l, i, 1].ToString().Replace(',', '.') + " Markierungen");
                }
                
                // output the results for the current looseness in gnuplot readable plotting files
                string fnameTime = "VarSize_Time_looseness_" + loosenesses[l] + ".txt";
                string fnameMarks = "VarSize_Marks_looseness_" + loosenesses[l] + ".txt";
                StreamWriter otime = new StreamWriter(fnameTime);
                StreamWriter omarks = new StreamWriter(fnameMarks);

                // write results for DFS
                otime.WriteLine("# Iterative DFS");
                omarks.WriteLine("# Iterative DFS");
                for (int s = sdiff, i = 0; s <= maxSize; s += sdiff, i++)
                {
                    otime.WriteLine(s + " " + times[l, i, 0].ToString().Replace(',', '.'));
                    omarks.WriteLine(s + " " + markCounts[l, i, 0].ToString().Replace(',', '.'));
                }
                otime.Write("\n\n");
                omarks.Write("\n\n");

                // write results for Turn Count
                otime.WriteLine("# Turn Count");
                omarks.WriteLine("# Turn Count");
                for (int s = sdiff, i = 0; s <= maxSize; s += sdiff, i++)
                {
                    otime.WriteLine(s + " " + times[l, i, 1].ToString().Replace(',', '.'));
                    omarks.WriteLine(s + " " + markCounts[l, i, 1].ToString().Replace(',', '.'));
                }

                otime.Close();
                omarks.Close();
            }
        }

        /// <summary>
        /// Experiment for the path finding algorithms on variable lossenesses for a defined labyrinth size.
        /// </summary>
        private void expVarLooseness()
        {
            System.Console.WriteLine("\n ===>  Starte Auswertung expVarLooseness()  <===");

            // measurement of time and marked rooms per looseness for one defined labyrinth size
            double[,] times = new double[maxLoose / ldiff + 1, 2];
            double[,] markCounts = new double[maxLoose / ldiff + 1, 2];
            long startTime, endTime;
            // for every looseness
            for (byte l = 0, i = 0; l <= maxLoose; l += ldiff, i++)
            {
                System.Console.WriteLine("-> Lockerheit: " + l);
                // go through all runs
                for (int run = 1; run <= runs; run++)
                {
                    System.Console.Write(run + ",");
                    // create labyrinth
                    Labyrinth lab = new Labyrinth(lsize, l);
                    LabPathFinder finder = new LabPathFinder(lab);

                    // make some DFS runs over this labyrinth
                    for (int j = 0; j < labruns; j++)
                    {
                        lab.UnmarkRooms();
                        startTime = timer.Value;
                        finder.FindWayDFSIt();
                        endTime = timer.Value;
                        times[i, 0] += (endTime - startTime) / (double)timer.Frequency;
                        markCounts[i, 0] += lab.CountMarkedRooms();
                    }

                    // make some TurnCounter runs over this labyrinth
                    for (int j = 0; j < labruns; j++)
                    {
                        lab.UnmarkRooms();
                        startTime = timer.Value;
                        finder.FindWayTurnCounter();
                        endTime = timer.Value;
                        times[i, 1] += (endTime - startTime) / (double)timer.Frequency;
                        markCounts[i, 1] += lab.CountMarkedRooms();
                    }
                }

                // calculate mean values
                times[i, 0] /= (double)(runs * labruns);
                times[i, 1] /= (double)(runs * labruns);
                markCounts[i, 0] /= (double)(runs * labruns);
                markCounts[i, 1] /= (double)(runs * labruns);

                System.Console.WriteLine();
                System.Console.WriteLine("DFS:       " + times[i, 0].ToString().Replace(',', '.') + "s, " + markCounts[i, 0].ToString().Replace(',', '.') + " Markierungen");
                System.Console.WriteLine("TurnCount: " + times[i, 1].ToString().Replace(',', '.') + "s, " + markCounts[i, 1].ToString().Replace(',', '.') + " Markierungen");
            }

            // output the results in gnuplot readable plotting files
            string fnameTime = "VarLoose_Time.txt";
            string fnameMarks = "VarLoose_Marks.txt";
            StreamWriter otime = new StreamWriter(fnameTime);
            StreamWriter omarks = new StreamWriter(fnameMarks);

            // write results for DFS
            otime.WriteLine("# Iterative DFS");
            omarks.WriteLine("# Iterative DFS");
            for (byte l = 0, i = 0; l <= maxLoose; l += ldiff, i++)
            {
                otime.WriteLine(l + " " + times[i, 0].ToString().Replace(',', '.'));
                omarks.WriteLine(l + " " + markCounts[i, 0].ToString().Replace(',', '.'));
            }
            otime.Write("\n\n");
            omarks.Write("\n\n");

            // write results for Turn Count
            otime.WriteLine("# Turn Count");
            omarks.WriteLine("# Turn Count");
            for (byte l = 0, i = 0; l <= maxLoose; l += ldiff, i++)
            {
                otime.WriteLine(l + " " + times[i, 1].ToString().Replace(',', '.'));
                omarks.WriteLine(l + " " + markCounts[i, 1].ToString().Replace(',', '.'));
            }

            otime.Close();
            omarks.Close();
        }
    }
}
