package algorithms.sorting;

public class SortingAlgorithmsInt {
	private int ops = 0;
	private int compares = 0;
	private long cmpIts = 0;
	
	// resets the operations and compares counter
	public void resetCounters(){
		ops = 0;
		compares = 0;
	}
	
	// resets the compare delay iterations
	public void resetCmpDelayIts(){
		cmpIts = 0;
	}
	
	// sets the compare delay iterations to "runs"
	public void setCmpDelayIts(long runs){
		cmpIts = runs;
	}
	
	// returns the compare delay iterations
	public long getCmpDelayIts(){
		return cmpIts;
	}
	
	// gets the number of operations from the last run
	public int getOps(){
		return ops;
	}
	
	// gets the number of compares from the last run
	public int getCompares(){
		return compares;
	}
	
	// normal quicksort
	public void qsort(int[] a) {
		qsort(a, 0, a.length-1);
	}

	// normal quicksort
	public void qsort(int[] a, int from, int to) {
		if(from < to){
			// partition magic
			int p=from-1, q=to;
			ops++;
			int pivot = a[to];
			while(p < q){
				while(compare(a[++p],pivot)<0);
				while(compare(a[--q],pivot)>0 && p<q);
				if(p < q)
					swap(a, p, q);
			}
			// swap pivot element to its final position
			swap(a, p, to);
			
			// recursive call for part arrays
			qsort(a, from, p-1);
			qsort(a, p+1, to);
		}
	}
	
	// iterative qsort combined with insertion sort
	public void qsortItISort(int[] a, int m) {
		qsortItISort(a, 0, a.length-1, m);
	}

	// iterative qsort combined with insertion sort
	public void qsortItISort(int[] a, int from, int to, int m) {
		while(to-from >= m){
			// partition magic
			int p=from-1, q=to;
			ops++;
			int pivot = a[to];
			while(p < q){
				while(compare(a[++p],pivot)<0);
				while(compare(a[--q],pivot)>0 && p<q);
				if(p < q)
					swap(a, p, q);
			}
			// swap pivot element to its final position
			swap(a, p, to);
			
			// do recursive call for the smaller par array
			if(p < (from+to)/2){
				qsortItISort(a, from, p-1, m);
				from = p+1;
			}
			else{
				qsortItISort(a, p+1, to, m);
				to = p-1;
			}
		}
		// insertion sort for little part arrays
		isort(a, from, to);
	}
	
	// iterative qsort
	public void qsortIt(int[] a) {
		qsortIt(a, 0, a.length-1);
	}

	// iterative qsort
	public void qsortIt(int[] a, int from, int to) {
		while(from < to){
			// partition magic
			int p=from-1, q=to;
			ops++;
			int pivot = a[to];
			while(p < q){
				while(compare(a[++p],pivot)<0);
				while(compare(a[--q],pivot)>0 && p<q);
				if(p < q)
					swap(a, p, q);
			}
			// swap pivot element to its final position
			swap(a, p, to);
			
			// do recursive call for the smaller part array
			if(p < (from+to)/2){
				qsortIt(a, from, p-1);
				from = p+1;
			}
			else{
				qsortIt(a, p+1, to);
				to = p-1;
			}
		}
	}
	
	// recursive qsort with insertion sort combined
	public void qsortISort(int[] a, int m) {
		qsortISort(a, 0, a.length-1, m);
	}

	// recursive qsort with insertion sort combined
	public void qsortISort(int[] a, int from, int to, int m) {
		if(to-from >= m){
			// partition magic
			int p=from-1, q=to;
			ops++;
			int pivot = a[to];
			while(p < q){
				while(compare(a[++p],pivot)<0);
				while(compare(a[--q],pivot)>0 && p<q);
				if(p < q)
					swap(a, p, q);
			}
			// swap pivot element to its final position
			swap(a, p, to);
			
			// recursive calls
			qsortISort(a, from, p-1, m);
			qsortISort(a, p+1, to, m);
		}
		// insertion sort for little part arrays
		else {
			isort(a, from, to);
		}
	}
	
	// recursive qsort + median-of-three
	public void qsortMo3(int[] a, int m) {
		qsortMo3(a, 0, a.length-1, m);
	}

	// recursive qsort + median-of-three
	public void qsortMo3(int[] a, int from, int to, int m) {
		if(from < to){
			// median-of-three
			if(to-from > 3){
				int mid = (from+to)/2;
				if(compare(a[from],a[mid]) > 0)
					swap(a, from, mid);
		  		if(compare(a[from],a[to]) > 0)
		  			swap(a, from, to);
		  		else if(compare(a[to],a[mid]) > 0)
		  			swap(a, mid, to);
			}
			
			// partition magic
			ops++;
			int p=from-1, q=to, pivot=a[to];
			
			while(p < q){
				while(compare(a[++p],pivot)<0);
				while(compare(a[--q],pivot)>0 && p<q);
				if(p < q)
					swap(a, p, q);
			}
			// swap pivot element to its final position
			swap(a, p, to);
			
			// recursive calls
			qsortMo3(a, from, p-1, m);
			qsortMo3(a, p+1, to, m);
		}
	}
	
	// recursive qsort + median-of-three + insertion sort
	public void qsortMo3ISort(int[] a, int m) {
		qsortMo3ISort(a, 0, a.length-1, m);
	}

	// recursive qsort + median-of-three + insertion sort
	public void qsortMo3ISort(int[] a, int from, int to, int m) {
		if(to-from >= m){
			// median-of-three
			if(to-from > 3){
				int mid = (from+to)/2;
				if(compare(a[from],a[mid]) > 0)
					swap(a, from, mid);
		  		if(compare(a[from],a[to]) > 0)
		  			swap(a, from, to);
		  		else if(compare(a[to],a[mid]) > 0)
		  			swap(a, mid, to);
			}
			
			// partition magic
			ops++;
			int p=from-1, q=to, pivot=a[to];
			
			while(p < q){
				while(compare(a[++p],pivot)<0);
				while(compare(a[--q],pivot)>0 && p<q);
				if(p < q)
					swap(a, p, q);
			}
			// swap pivot element to its final position
			swap(a, p, to);
			
			// recursive calls
			qsortMo3ISort(a, from, p-1, m);
			qsortMo3ISort(a, p+1, to, m);
		}
		// insertion sort for little part arrays
		else { 
			isort(a, from, to);
		}
	}
	
	// insertion sort
	public void isort(int[] a) {
		isort(a, 0, a.length-1);
	}

	// insertion sort
	public void isort(int[] a, int from, int to) {
		for(int i=from+1; i<=to; i++){
			ops++;
			int ins = a[i];
			int k;
			for(k=i; k>0 && compare(a[k-1],ins)>0; k--){
				ops++;
				a[k] = a[k-1];
			}
			ops++;
			a[k] = ins;
		}
	}
	
	// normal heapsort
	public void heapsort(int[] a) {
		heapsort(a, 0, a.length-1);
	}

	// normal heapsort
	public void heapsort(int[] a, int from, int to) {
		// create heap order
		createMaxHeap(a);
		// one run: set highest value to end and restore heap order 
		for(int i=a.length-1; i>1; i--){
			swap(a, 0, i);
			sinkMaxHeap(a, 0, i-1);
		}
		swap(a, 0, 1);
	}
	
	// creates the max heap
	private void createMaxHeap(int[] a){
		// sink nodes in the tree to get a heap order; O(n log n)
		for(int i=(a.length-1)/2; i>=0; i--)
			sinkMaxHeap(a, i, a.length-1);
	}
	
	// let a[i] sink into the max heap
	private void sinkMaxHeap(int[] a, int i, int r){
		// left son is greater
		int ls = 2*(i+1)-1;
		int max = (ls<=r && compare(a[ls],a[i])>0) ? ls : i;
		// right son is greater
		int rs = 2*(i+1);
		if(rs<=r && compare(a[rs],a[max])>0)
			max = rs;
		// if one son was greater -> sink node i recursively
		if(max != i){
			swap(a, i, max);
			sinkMaxHeap(a, max, r);
		}
	}
	
	// bottom-up-heapsort
	public void bottomUpHeapsort(int[] a) {
		bottomUpHeapsort(a, 0, a.length-1);
	}

	// bottom-up-heapsort
	public void bottomUpHeapsort(int[] a, int from, int to) {
		// create heap order
		createMaxHeap(a);
		// one run: set highest value to end and determine next highest value 
		for(int i=a.length-1; i>1; i--){
			swap(a, 0, i);
			bottomUpSinking(a, i-1);
		}
		swap(a, 0, 1);
	}
	
	// let a[0] sink into the heap
	private void bottomUpSinking(int[] a, int r){
		ops++;
		int elem = a[0];     // a[0] is to be overwritten
		int j = 0;           // index of current node
		int m = 1;           // index of left son node
		// search for the path of highest son values starting at sons of the root m,m+1 
		while(m < r){
			// set current node to maximum of left and right son
			int max = (compare(a[m],a[m+1]) < 0) ? m+1 : m;
			ops++;
			a[j] = a[max];
			j = max;
			// max node of left and right son is next
			m = 2*(max+1)-1;
		}
		
		// if current left son is right border -> copy to parent
		if(m == r){
			ops++;
			a[j] = a[r];
			j = m;
		}
		
		// go from bottom to top - search for insert position for last array elem
		int i = (j-1)/2;
		while(j>0 && compare(a[i],elem)<0){
			ops++;
			a[j] = a[i];
			j = i;
			i = (j-1)/2;
		}
		ops++;
		a[j] = elem;
	}
	
	// swap a[i] with a[j]
	private void swap(int[] a, int i, int j){
		ops += 3;
		int tmp = a[i];
		a[i] = a[j];
		a[j] = tmp;
	}
	
	// comparison method for two elements
	private int compare(int a, int b){
		// if compare iterations have to be done -> run a while loop over these
		if(cmpIts > 0){
			int i = 0;
			while(i < cmpIts)
				i++;
		}
		// increment compares counter
		compares++;
		// return the comparison result
		return (a<b) ? -1 : (a>b) ? 1 : 0;
	}
}
